﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using com.jnbridge.jnbproxy;
using com.jnbridge.labs.visio;
using org.apache.hadoop.hdfs.server.namenode;
using org.apache.hadoop.hdfs.protocol;
using org.apache.hadoop.mapred;
using org.apache.hadoop.net;

// HadoopTree.cs
// Code for building and updating the internal representation of the Hadoop Cluster

namespace HadoopVisProto
{
    class HadoopTree
    {
        private static HadoopNode nnHead;
        private static HadoopNode jtHead;
        private static Dictionary<string, HadoopNode> innerNodes;
        private static Dictionary<string, HadoopNode> leaves;

        public static void buildTree(JobTracker jobTracker, string jtHostname, string nnHostName, Object[] taskNodeInfo, DatanodeInfo[] dataNodeInfo)
        {
            innerNodes = new Dictionary<string, HadoopNode>();
            leaves = new Dictionary<string, HadoopNode>();

            // Build hdfs side
            nnHead = new HadoopNode(0, nnHostName);

            // Scan through datanode data and build tree
            foreach (DatanodeInfo dn in dataNodeInfo)
            {
                string location = dn.getNetworkLocation();
                string[] parsedLocation = location.Split(new string[1] {"/"}, StringSplitOptions.RemoveEmptyEntries);

                // Build inner nodes
                HadoopNode parent = nnHead;
                foreach (string s in parsedLocation)
                {
                    if (innerNodes.ContainsKey(s))
                    {
                        HadoopNode newParent;
                        innerNodes.TryGetValue(s,out newParent);
                        System.Diagnostics.Debug.Assert(newParent.getHDFSParent().Equals(parent));
                        parent = newParent;
                    }
                    else
                    {
                        HadoopNode newNode = new HadoopNode(1, s);
                        newNode.setHDFSParent(parent);
                        parent.addChild(newNode);
                        parent = newNode;
                        innerNodes.Add(newNode.name, newNode);
                    }
                }
                // Build Leaf
                HadoopNode newLeaf = new HadoopNode(2, dn.getHostName());
                parent.addChild(newLeaf);
                newLeaf.setHDFSParent(parent);
                newLeaf.markAsDataNode();
                if (!dn.isDecommissioned())
                {
                    newLeaf.dataActive = true;
                }
                leaves.Add(newLeaf.name, newLeaf);
                
            }

            // Build mapred side 
            jtHead = new HadoopNode(0, jtHostname);
            JavaSides.setJavaServer("default");
            // Scan through tasktracker data and augment tree
            foreach (TaskTrackerStatus tt in taskNodeInfo)
            {
                string name = tt.getHost();
                string location = jobTracker.getNode(name).getNetworkLocation();
                string[] parsedLocation = location.Split(new string[1] { "/" }, StringSplitOptions.RemoveEmptyEntries);

                // Build inner nodes if not already
                HadoopNode parent = jtHead;
                foreach (string s in parsedLocation)
                {
                    if (innerNodes.ContainsKey(s))
                    {
                        HadoopNode newParent;
                        innerNodes.TryGetValue(s, out newParent);
                        if (newParent.getMapredParent() == null)
                        {
                            newParent.setMapredParent(parent);
                        }
                        else
                        {
                            System.Diagnostics.Debug.Assert(newParent.getMapredParent().Equals(parent));
                        } 
                        parent = newParent;
                    }
                    else
                    {
                        HadoopNode newNode = new HadoopNode(1, s);
                        newNode.setMapredParent(parent);
                        parent.addChild(newNode);
                        parent = newNode;
                        innerNodes.Add(newNode.name, newNode);
                    }
                }
                // Build Leaf of mark existing leaves as tasktrackers
                if (leaves.ContainsKey(name))
                {
                    HadoopNode extantLeaf;
                    leaves.TryGetValue(name, out extantLeaf);
                    extantLeaf.setMapredParent(parent);
                    extantLeaf.markAsTaskTracker();
                    extantLeaf.taskActive = true;
                }
                else
                {
                    HadoopNode newLeaf = new HadoopNode(1, name);
                    newLeaf.setMapredParent(parent);
                    parent.addChild(newLeaf);
                    newLeaf.markAsTaskTracker();
                    newLeaf.taskActive = true;
                    leaves.Add(newLeaf.name, newLeaf);
                }
            }
            VisioDrawer.createPage();
            VisioDrawer.drawTree(jtHead, nnHead, innerNodes, leaves);
        }

        public static void updateTree(Object[] taskNodeInfo, Object[] deadTaskNodes, DatanodeInfo[] dataNodeInfo)
        {
            JavaSides.setJavaServer("NameNode");
            foreach (DatanodeInfo dn in dataNodeInfo)
            {
                HadoopNode curNode;
                leaves.TryGetValue(dn.getHostName(), out curNode);
                if (dn.isDecommissioned())
                {
                    curNode.dataActive = false;
                }
                else
                {
                    curNode.setHDSpace(dn.getRemainingPercent());
                    curNode.dataActive = true;
                }
            }
            JavaSides.setJavaServer("default");
            foreach (TaskTrackerStatus tt in taskNodeInfo)
            {
                HadoopNode curNode;
                leaves.TryGetValue(tt.getHost(), out curNode);
                curNode.setMapUse(tt.getMaxMapSlots(), tt.countOccupiedMapSlots());
                curNode.setReduceUse(tt.getMaxReduceSlots(), tt.countOccupiedReduceSlots());
                curNode.taskActive = true;
            }
            foreach (TaskTrackerStatus tt in deadTaskNodes)
            {
                HadoopNode curNode;
                leaves.TryGetValue(tt.getHost(), out curNode);
                curNode.taskActive = false;
            }
            VisioDrawer.updateData(leaves);
        }
    }
}
